/*

The Clans BBS Door Game
Copyright (C) 1997-2002 Allen Ussher

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// Player Editor for The Clans -- no promises that this works! 01/09/2002 au

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <time.h>
#include <string.h>
#include "opendoor.h"
#include "structs.h"

struct Village Village;

void InitVillage ( void );
void UpdateVillage ( void );
void CheckMem ( void *Test );
void RejectTrade ( struct TradeData *TradeData );
void GetAlliances( struct Alliance *Alliances[MAX_ALLIANCES]);
void UpdateAlliances( struct Alliance *Alliances[MAX_ALLIANCES]);
void RemoveFromIPScores ( int ClanID[2] );
BOOL ClanIDInList( int ClanID[2] );
void RemoveFromUList(int ClanID[2]);
void FreeClan( struct Clan *Clan );
BOOL GetClan ( int ClanID[2], struct Clan *TmpClan );
void DeleteClan (int ClanID[2]);

void main ( void )
{
    FILE *fpPC;
    long FileSize;
    int NumClans, CurClan;
    char cKey;
    struct Clan Clan;

    printf(".PC (Player Clans) Editor for The Clans.  v0.10\n");
    printf("\nThis PC editor should only be used on local games and only when\n");
    printf("no one is currently playing.\n\n");
    printf("Continue? [Yes]: ");

    do {
        cKey = toupper(getch());
    } while (!strchr("YN\r", cKey));

    if (cKey == 'N')
    {
        printf("No\n\nProgram Aborted\n");
        exit(0);
    }
    else
        printf("Yes\n\n");

    // WARNING:  Do not use while player is online!!

    // if no PC file, boot to DOS

    fpPC = fopen("CLANS.PC","rb");

    if (!fpPC)
    {
        printf("No CLANS.PC file!\n");
        exit(0);
    }

    // otherwise, open file, get filesize
    fseek(fpPC, 0L, SEEK_END);
    FileSize = ftell(fpPC);
    NumClans = FileSize/(sizeof(struct Clan) + 6*sizeof(struct PC));

    // open Village Data
    InitVillage();

    CurClan = 0;

    for (;;)
    {
        if (fseek(fpPC, CurClan*( sizeof(struct Clan) + 6*sizeof(struct PC) ), SEEK_SET))
        {
            printf("fseek error\n");
            break;
        }

        // get curuser from file
        if (!fread(&Clan, sizeof(struct Clan), 1, fpPC))
        {
            printf("Couldn't read any players\n");
            break;
        }

        // display user stats
        printf("ClanID  : %02d|%02d\n", Clan.ClanID[0], Clan.ClanID[1]);
        printf("Clan    : %s\n", Clan.szName);
        printf("User    : %s\n", Clan.szMasterName);
        printf("LastPlay: %s\n", Clan.LastDate);

        printf("\n\n ] next\n [ previous\n ! delete clan\n Q quit\n---------------------\n# ");

        // get key
        do {
            cKey = getch();
        } while (!strchr("[]q!", cKey));

        if (cKey == 'q')
        {
            printf("Quit\n\n");
            break;
        }
        else if (cKey == '[')
        {
            printf("Previous Clan\n\n");

            if (CurClan > 0)
                CurClan--;
            else
            {
                CurClan = NumClans-1;
            }
        }
        else if (cKey == ']')
        {
            printf("Next Clan\n\n");

            if (CurClan < NumClans-1)
                CurClan++;
            else
                CurClan = 0;
        }
        else if (cKey == '!')
        {
            printf("Delete Clan\n\n");

            fclose(fpPC);

            printf("Deleting %s\n", Clan.szName);

            DeleteClan(Clan.ClanID);

            fpPC = fopen("CLANS.PC","r+b");

            if (!fpPC)
            {
                printf("No CLANS.PC file!\n");
                exit(0);
            }

            // otherwise, open file, get filesize
            fseek(fpPC, 0L, SEEK_END);
            FileSize = ftell(fpPC);
            NumClans = FileSize/(sizeof(struct Clan) + 6*sizeof(struct PC));

            if (NumClans == 0)
            {
                printf("All players deleted\n");
                break;
            }

            CurClan = 0;

            UpdateVillage();
        }

        // [ = back one user
            // if at user #1, figure out last user using FileSize/sizeof()
            // choose that user as one to edit
            // else if > #1, CurUser--

        // ] = forward a user
            // if already at end, CurUser = 0
            // else

        // ! = delete user
            // close .PC file
            // run DeleteClan
            // reload file and get stats once more
            // update village info now in case crash occurs later

    }

    fclose(fpPC);

    // Update Village Data
    UpdateVillage();
}

void DeleteClan ( int ClanID[2])
{
	FILE *fpOldPC, *fpNewPC, *OldMessage, *NewMessage;
	FILE *fpTradeFile;
	long OldOffset;
	char szFileName[40], szString[128];
    int CurTradeData, iTemp, CurAlliance, CurMember;
	struct TradeData TradeData;
	struct Clan *TmpClan;
	struct Message Message;
	BOOL FoundInPCFile = FALSE; 	// set to true if he was ever on this board
	BOOL FoundNewCreator;
	struct Alliance *Alliances[MAX_ALLIANCES];

	// if this is the ruler of town, remove him
	if (ClanID[0] == Village.RulingClanId[0] &&
		ClanID[1] == Village.RulingClanId[1])
	{
		Village.RulingClanId[0] = -1;
		Village.RulingClanId[1] = -1;
		Village.RulingDays = 0;
        Village.GovtSystem = GS_DEMOCRACY;

		Village.szRulingClan[0] = 0;

		UpdateVillage();
	}

	// go through PC file
    fpOldPC = fopen("CLANS.PC", "rb");
	if (fpOldPC)
	{
        fpNewPC = fopen("NEW.PC", "w+b");
		if (!fpNewPC)
		{
            printf("Can't write to NEW.PC!\n");
			exit(0);
		}

		/* allocate memory */
		TmpClan = (struct Clan *) malloc(sizeof(struct Clan));
		CheckMem(TmpClan);

		for (;;)
		{
			/* go through each clan and write his info to new file and
			   skip the clan in question if he is found */

			if (fread(TmpClan, sizeof(struct Clan), 1, fpOldPC) == 0)
				break;

			/* if this is him */
			if (TmpClan->ClanID[0] == ClanID[0] &&
				TmpClan->ClanID[1] == ClanID[1])
			{
                FoundInPCFile = TRUE;

                fseek(fpOldPC, 6L*sizeof(struct PC), SEEK_CUR);
                continue;
			}

			// read in 6 members
			for (iTemp = 0; iTemp < 6; iTemp++)
			{
				TmpClan->Member[iTemp] = malloc(sizeof(struct PC));
				fread(TmpClan->Member[iTemp], sizeof(struct PC), 1, fpOldPC);
			}

			//=== modifications go here
            // make it so that he is NOT voted for
            if (TmpClan->ClanRulerVote[0] == ClanID[0] &&
                TmpClan->ClanRulerVote[1] == ClanID[1])
            {
                TmpClan->ClanRulerVote[0] = TmpClan->ClanRulerVote[1] = -1;
            }


			// IN FUTURE: if ruler of an alliance, set new ruler if another
			// member was found.  if none, remove alliance from list

			//===

			/* write new stuff to new file */
			fwrite(TmpClan, sizeof(struct Clan), 1, fpNewPC);

			for (iTemp = 0; iTemp < 6; iTemp++)
			{
				fwrite(TmpClan->Member[iTemp], sizeof(struct PC), 1, fpNewPC);
				free(TmpClan->Member[iTemp]);
			}
		}

		/* deallocate memory for PCs */
		free(TmpClan);

		fclose(fpOldPC);
		fclose(fpNewPC);

		/* delete old file, rename new one */
        unlink("CLANS.PC");
        rename("NEW.PC", "CLANS.PC");
	}

	// go through msg file, set all his mail (to/from him) as deleted

    strcpy(szFileName, "CLANS.MSJ");

	OldMessage = fopen(szFileName, "rb");
	if (OldMessage) 		// MSJ file exists, so go on
	{
		NewMessage = fopen("TMP.$$$", "wb");
		if (!NewMessage)
		{
			return;
		}

		for (;;)
		{
			if (!fread(&Message, sizeof(struct Message), 1, OldMessage))
				break;

			if ((Message.FromClanId[0] == ClanID[0] &&
				 Message.FromClanId[1] == ClanID[1]) ||
				(Message.ToClanId[0] == ClanID[0] &&
				 Message.ToClanId[1] == ClanID[1]) )
			{
				// delete this message by skipping over it
				fseek(OldMessage, Message.Length, SEEK_CUR);
			}
			else
			{
				Message.MsgTxt = malloc(Message.Length);
				CheckMem(Message.MsgTxt);

				// write it to new file
				fread(Message.MsgTxt, Message.Length, 1, OldMessage);

				fwrite(&Message, sizeof(Message), 1, NewMessage);
				fwrite(Message.MsgTxt, Message.Length, 1, NewMessage);

				free(Message.MsgTxt);
			}
		}

		fclose(NewMessage);
		fclose(OldMessage);

		// delete old, and rename new
		unlink(szFileName);
		rename("TMP.$$$", szFileName);
	}

	// go through trades.dat, for each trade struct, see if trade is headed
	// for him, if so, set trade as aborted, return goods to user sending

	fpTradeFile = fopen("TRADES.DAT", "r+b");

	if (fpTradeFile)
	{
		for (CurTradeData = 0;;CurTradeData++)
		{
			if (fseek(fpTradeFile, (long)(CurTradeData * sizeof(struct TradeData)), SEEK_SET))
				break;

			OldOffset = ftell(fpTradeFile);

			if (fread(&TradeData, sizeof(struct TradeData), 1, fpTradeFile) == 0)
				break;

			/* see if active */
			if (TradeData.Active == FALSE)
				continue;

            if (TradeData.ToClanId[0] == ClanID[0] &&
				TradeData.ToClanId[1] == ClanID[1])
			{
				// it's for this deleted player, so, rejecttrade
                RejectTrade(&TradeData);

				// write it to file
				fseek(fpTradeFile, OldOffset, SEEK_SET);
				fwrite(&TradeData, sizeof(struct TradeData), 1, fpTradeFile);
			}
            else if (TradeData.FromClanId[0] == ClanID[0] &&
				TradeData.FromClanId[1] == ClanID[1])
			{
				// trade is coming from this player, remove it
				TradeData.Active = FALSE;
				fseek(fpTradeFile, OldOffset, SEEK_SET);
				fwrite(&TradeData, sizeof(struct TradeData), 1, fpTradeFile);
			}
		}

		fclose(fpTradeFile);
	}


    // remove from ALLY.DAT
    GetAlliances(Alliances);

	// see if this clan is the creator of an alliance
	for (CurAlliance = 0; CurAlliance < MAX_ALLIANCES; CurAlliance++)
	{
		if (Alliances[CurAlliance] &&
			Alliances[CurAlliance]->CreatorID[0] == ClanID[0] &&
			Alliances[CurAlliance]->CreatorID[1] == ClanID[1])
		{
			// find a new "leader"
			FoundNewCreator = FALSE;
			for (CurMember = 0; CurMember < MAX_ALLIANCEMEMBERS; CurMember++)
			{
				// if this is an actual member AND it's not the deleted clan,
				// see if
				if ((Alliances[CurAlliance]->Member[CurMember][0] != -1 &&
					 Alliances[CurAlliance]->Member[CurMember][1] != -1) &&
					!(Alliances[CurAlliance]->Member[CurMember][0] == ClanID[0] &&
					 Alliances[CurAlliance]->Member[CurMember][1] == ClanID[1]))
				{
					FoundNewCreator = TRUE;
					Alliances[CurAlliance]->CreatorID[0] = Alliances[CurAlliance]->Member[CurMember][0];
					Alliances[CurAlliance]->CreatorID[1] = Alliances[CurAlliance]->Member[CurMember][1];

					// FIXME: write him a message in future?
				}
			}

			if (FoundNewCreator == FALSE)
			{
				// delete this alliance since no new ruler
				free(Alliances[CurAlliance]);
				Alliances[CurAlliance] = NULL;
			}
		}
	}

	// remove from any and all alliances he was in
	for (CurAlliance = 0; CurAlliance < MAX_ALLIANCES; CurAlliance++)
	{
		for (CurMember = 0; CurMember < MAX_ALLIANCEMEMBERS; CurMember++)
		{
			if (Alliances[CurAlliance]->Member[CurMember][0] == ClanID[0] &&
				Alliances[CurAlliance]->Member[CurMember][1] == ClanID[1])
			{
				Alliances[CurAlliance]->Member[CurMember][0] = -1;
				Alliances[CurAlliance]->Member[CurMember][1] = -1;
			}
		}
	}

	// deinit alliances and update to file
    UpdateAlliances(Alliances);

	// free up mem used by alliances
	for (CurAlliance = 0; CurAlliance < MAX_ALLIANCES; CurAlliance++)
		if (Alliances[CurAlliance])
			free(Alliances[CurAlliance]);

	// remove from list of clan names, remove from list of user names
    RemoveFromUList(ClanID);

    // remove from high scores list
    RemoveFromIPScores(ClanID);
}

void InitVillage ( void )
{
	FILE *fpVillage;
    char ColorScheme[23] = {
            6,14, 7, 5, 13,7, 5, 8, 3, 8,     5, 14,6, 4, 12,   4,12,3,
            6,0,0,  1, 9};
	char FlagScheme[3] = { 12, 15, 9 };
	int iTemp;

	/* try opening it for share */
    fpVillage = fopen("VILLAGE.DAT", "rb");
	if (!fpVillage)
	{
        printf("Error opening VILLAGE.DAT!\n");
        exit(0);
	}
	else
		fread(&Village, sizeof(struct Village), 1, fpVillage);

	fclose(fpVillage);
}

void UpdateVillage ( void )
{
	FILE *fpVillage;

	/* NOTE: Village data held in memory (global) is NOT accurate if this game
	   is played in multinode since the other node could write new info to
	   the village.dat.  Only use Village struct for reference and update it
	   if you want to display info */

	/* try opening it for share */
    fpVillage = fopen("VILLAGE.DAT", "wb");
	if (!fpVillage)
	{
        printf("Error opening VILLAGE.DAT!\n");
        exit(0);
	}

	fwrite(&Village, sizeof(struct Village), 1, fpVillage);

	fclose(fpVillage);
}

void CheckMem ( void *Test )
{
	if (Test == NULL)
	{
        printf("\aCheckmem Failed!\n");
		exit(-1);
	}
}

void RejectTrade ( struct TradeData *TradeData )
{
    struct Clan *TmpClan;

    TradeData->Active = FALSE;

    TmpClan = malloc(sizeof(struct Clan));
    CheckMem(TmpClan);

    GetClan(TradeData->FromClanId, TmpClan);

    TmpClan->Empire.VaultGold += TradeData->Giving.Gold;
    TmpClan->Empire.Army.Followers += TradeData->Giving.Followers;
    TmpClan->Empire.Army.Footmen += TradeData->Giving.Footmen;
    TmpClan->Empire.Army.Axemen += TradeData->Giving.Axemen;
    TmpClan->Empire.Army.Knights += TradeData->Giving.Knights;

    /* this ensures the file is zeroed for cheaters -- improve later */
    TradeData->Giving.Gold = 0L;
    TradeData->Giving.Followers = 0L;
    TradeData->Giving.Footmen = 0L;
    TradeData->Giving.Axemen = 0L;
    TradeData->Giving.Knights = 0L;
    TradeData->Giving.Catapults = 0L;

    UpdateClan(TmpClan);

    FreeClan(TmpClan);
}

void GetAlliances( struct Alliance *Alliances[MAX_ALLIANCES])
{
	FILE *fp;
	int iTemp;

	// init alliances as NULLs
	for (iTemp = 0; iTemp < MAX_ALLIANCES; iTemp++)
		Alliances[iTemp] = NULL;

	fp = fopen("ALLY.DAT", "rb");
	if (fp)
	{
		for (iTemp = 0; iTemp < MAX_ALLIANCES; iTemp++)
		{
			Alliances[iTemp] = malloc(sizeof(struct Alliance));
			CheckMem(Alliances[iTemp]);

			if (fread(Alliances[iTemp], sizeof(struct Alliance), 1, fp) == 0)
			{
				// no more alliances to read in
				free(Alliances[iTemp]);
				Alliances[iTemp] = NULL;
				break;
			}
		}
		fclose(fp);
	}
}

void UpdateAlliances( struct Alliance *Alliances[MAX_ALLIANCES])
{
	FILE *fp;
	int iTemp;

	fp = fopen("ALLY.DAT", "wb");
	if (fp)
	{
		for (iTemp = 0; iTemp < MAX_ALLIANCES; iTemp++)
		{
			if (Alliances[iTemp] == NULL)
				continue;

			fwrite(Alliances[iTemp], sizeof(struct Alliance), 1, fp);
		}
		fclose(fp);
	}
}

void RemoveFromUList(int ClanID[2])
{
    // open file for r+b
    // scan file for ClanID
    // set user as deleted
    // close file

    FILE *fpOldUList, *fpNewUList;
    struct UserInfo User;

    // if this guy is NOT in our list, then we don't need to do this
    if (ClanIDInList(ClanID) == FALSE)
        return;

    // open user file
    fpOldUList = fopen("USERLIST.DAT", "rb");
    if (!fpOldUList)    // no user list at all
        return;

    fpNewUList = fopen("TMP.$$$", "wb");
    // FIXME: assume file is opened

    for (;;)
    {
        if (fread(&User, sizeof(struct UserInfo), 1, fpOldUList) == NULL)
            break;

//        printf("Read in %s\n", User.szName);

        // for each user in file, see if same as ClanID
        if (User.ClanID[0] == ClanID[0] && User.ClanID[1] == ClanID[1])
        {
            //printf("skipping over %s\n", User.szName);
            // same, skip over him
            continue;
        }

        // otherwise, don't skip him, write him to new file
        fwrite(&User, sizeof(struct UserInfo), 1, fpNewUList);
    }

    // close file
    fclose(fpOldUList);
    fclose(fpNewUList);

    // rename file
    unlink("USERLIST.DAT");
    rename("TMP.$$$", "USERLIST.DAT");
}

BOOL ClanIDInList( int ClanID[2] )
{
    FILE *fpUList;
    BOOL Found = FALSE;
    struct UserInfo User;

    fpUList = fopen("USERLIST.DAT", "rb");
    if (!fpUList)
    {
        // no user list found, assume not in list then
        return FALSE;
    }

    // scan through file until end
    for (;;)
    {
        if (fread(&User, sizeof(struct UserInfo), 1, fpUList) == 0)
            break;

        // see if this user's name is same as one we're looking for
        if (User.ClanID[0] == ClanID[0] &&
            User.ClanID[1] == ClanID[1])
        {
                Found = TRUE;
                break;
        }
    }

    fclose(fpUList);

    return Found;
}

void RemoveFromIPScores ( int ClanID[2] )
{
    struct UserScore **ScoreList;
    int iTemp, UsersFound = 0;
    char ScoreDate[11];
    FILE *fp;

    fp = fopen("IPSCORES.DAT", "rb");

    if (!fp)
    {
        return;
    }

    // initialize the score data
    ScoreList = malloc(sizeof(struct UserScore *)*MAX_USERS);
    CheckMem(ScoreList);

    for (iTemp = 0; iTemp < MAX_USERS; iTemp++)
        ScoreList[iTemp] = NULL;

    // read date
    fread(ScoreDate, 11, 1, fp);

    for (iTemp = 0; iTemp < MAX_USERS; iTemp++)
    {
        ScoreList[iTemp] = malloc(sizeof(struct UserScore));
        CheckMem(ScoreList[iTemp]);
        if (!fread(ScoreList[iTemp], sizeof(struct UserScore), 1, fp))
        {
            free(ScoreList[iTemp]);
            ScoreList[iTemp] = NULL;
            break;
        }

        // if this is the user we wanted, remove him from the list
        if (ScoreList[iTemp]->ClanID[0] == ClanID[0] &&
            ScoreList[iTemp]->ClanID[1] == ClanID[1])
        {
            free(ScoreList[iTemp]);
            ScoreList[iTemp] = NULL;
        }
    }
    UsersFound = iTemp;
    fclose(fp);

    fp = fopen("IPSCORES.DAT", "wb");

    // write date
    fwrite(ScoreDate, 11, 1, fp);

    // write them to file now and free them at the same time
    for (iTemp = 0; iTemp < MAX_USERS; iTemp++)
        if (ScoreList[iTemp])
        {
            fwrite(ScoreList[iTemp], sizeof(struct UserScore), 1, fp);
            free(ScoreList[iTemp]);
        }

    free(ScoreList);

    fclose(fp);
}

void FreeClan( struct Clan *Clan )
{
	int CurMember;

	// free members first
	for (CurMember = 0; CurMember < MAX_MEMBERS; CurMember++)
	{
		if (Clan->Member[CurMember])
		{
			free(Clan->Member[CurMember]);
			Clan->Member[CurMember] = NULL;
		}
	}

	free(Clan);
	Clan = NULL;
}

BOOL GetClan ( int ClanID[2], struct Clan *TmpClan )
{
	FILE *fpPlayerFile;
	int ClanNum, iTemp;
	char szFileName[50];

    // make them all NULLs for safety
    for (iTemp = 0; iTemp < MAX_MEMBERS; iTemp++)
        TmpClan->Member[iTemp] = NULL;

    strcpy(szFileName, "CLANS.PC");

	/* find guy in file */
	fpPlayerFile = fopen(szFileName, "rb");
	if (!fpPlayerFile)
	{
        return FALSE;     /* means failed to find clan */
	}

	for (ClanNum = 0;; ClanNum++)
	{
        if (fseek(fpPlayerFile, (long)ClanNum * (sizeof(struct Clan) + 6L*sizeof(struct PC)), SEEK_SET))
		{
            // couldn't find clan in file
			fclose(fpPlayerFile);
            return FALSE;
		}
        if (!fread(TmpClan, sizeof(struct Clan), 1, fpPlayerFile))
        {
			fclose(fpPlayerFile);
            return FALSE;
        }

        // make them all NULLs for safety
        for (iTemp = 0; iTemp < MAX_MEMBERS; iTemp++)
            TmpClan->Member[iTemp] = NULL;

		if (TmpClan->ClanID[0] == ClanID[0] &&
			TmpClan->ClanID[1] == ClanID[1])
		{
			/* found it */
			/* read in PCs */
			for (iTemp = 0; iTemp < 6; iTemp++)
			{
				TmpClan->Member[iTemp] = malloc(sizeof(struct PC));
                CheckMem(TmpClan->Member[iTemp]);
				fread(TmpClan->Member[iTemp], sizeof(struct PC), 1, fpPlayerFile);

                /* skip those members which are non-existant */
                if (TmpClan->Member[iTemp]->szName[0] == 0)
                {
                    free(TmpClan->Member[iTemp]);
                    TmpClan->Member[iTemp] = NULL;
                }
                else
                    TmpClan->Member[iTemp]->MyClan = TmpClan;
			}
			break;
		}
	}
	fclose(fpPlayerFile);

    return TRUE;
}

void UpdateClan ( struct Clan *Clan )
{
    FILE *fpPlayerFile;
    char szFileName[50];
    int CurClan, iTemp;
    long OldOffset, Offset;
    struct Clan *TmpClan;
    struct PC *TmpPC;

    TmpClan = malloc(sizeof(struct Clan));
    CheckMem(TmpClan);
    TmpPC = malloc(sizeof(struct PC));
    CheckMem(TmpPC);

    fpPlayerFile = fopen("CLANS.PC", "r+b");
    if (!fpPlayerFile)
    {
        printf("Couldn't open CLANS.PC\n");
        free(TmpClan);
        free(TmpPC);
        return;  /* failed to find clan */
    }

    for (CurClan = 0;; CurClan++)
    {
        /* go through file till you find clan he wants */

        Offset = (long)CurClan * (sizeof(struct Clan) + 6L*sizeof(struct PC));
        if (fseek(fpPlayerFile, Offset, SEEK_SET))
        {
			break;	/* couldn't fseek, so exit */
        }

        OldOffset = ftell(fpPlayerFile);

        if (fread(TmpClan, sizeof(struct Clan), 1, fpPlayerFile) == 0)
			break;	/* stop reading if no more players found */

        /* skip if deleted clan */
        if (TmpClan->ClanID[0] == -1)
            continue;

        /* if same Ids, seek back and write to file */
        if (TmpClan->ClanID[0] == Clan->ClanID[0] &&
            TmpClan->ClanID[1] == Clan->ClanID[1])
        {
            fseek(fpPlayerFile, OldOffset, SEEK_SET);
            fwrite(Clan, sizeof(struct Clan), 1, fpPlayerFile);

            // fwrite players
            TmpPC->szName[0] = 0;
            for (iTemp = 0; iTemp < 6; iTemp++)
            {
                if (Clan->Member[iTemp] && Clan->Member[iTemp]->Undead == FALSE)
                    fwrite(Clan->Member[iTemp], sizeof(struct PC), 1, fpPlayerFile);
                else
                    fwrite(TmpPC, sizeof(struct PC), 1, fpPlayerFile);
            }
        }
    }
    fclose(fpPlayerFile);

    free(TmpPC);
    free(TmpClan);
}
