/*

The Clans BBS Door Game
Copyright (C) 1997-2002 Allen Ussher

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <dos.h>
#include <mem.h>
#include <ctype.h>
#include <malloc.h>
#include <share.h>

#include "structs.h"
#include "parsing.h"
#include "k_config.h"
#include "myopen.h"
#include "crc.h"

#define MAX_OPTION		14

#define K_UP        72
#define K_DOWN			80
#define K_HOME			71
#define K_END       79
#define K_PGUP			73
#define K_PGDN			81
#define K_LEFT			75
#define K_RIGHT 		77

#define HILIGHT     11|(1<<4)

#define COLOR	0xB800
#define MONO	0xB000

/* reset types */
#define RESET_LOCAL     1   // normal reset -- local, NO ibbs junk
#define RESET_JOINIBBS  2   // joining an IBBS league game
#define RESET_LC        3   // leaguewide reset by LC


#define FIGHTSPERDAY	12
#define CLANCOMBATADAY	5


void DosGetStr(char *InputStr, int MaxChars);
char get_answer( char *szAllowableChars );
void qputs (char *string, int x, int y);
void xputs (char *string, int x, int y);
void Video_Init ( void );
void ResetMenu( BOOL InterBBSGame );
void zputs (char *string);

int NoYes( char *Query );
int YesNo( char *Query );

void EditOption ( int WhichOption );
void DosHelp ( char *Topic, char *File );
long DosGetLong( char *Prompt, long DefaultVal, long Maximum );
void ColorArea( int xPos1, int yPos1, int xPos2, int yPos2, char Color );
void GoReset ( struct ResetData *ResetData, int Option );

void Config_Init ( void );
void Config_Close ( void );

void News_AddNews(char *szString);
void News_CreateTodayNews ( void );

void CreateVillageDat ( struct ResetData *ResetData );
void KillAlliances( void );

void System_Error ( char *szErrorMsg );

struct {
  long VideoType;
  long y_lookup[25];
  char far *VideoMem;
} Video;

struct ResetData ResetData;

struct config *Config;
struct game Game = { FALSE, NULL };

char szTodaysDate[11];

//extern unsigned _stklen = 32U*(1024U);


void main ( void )
{
  struct date date;

  Video_Init();
  Config_Init();

  // initialize date
  getdate(&date);
  sprintf(szTodaysDate, "%02d/%02d/%4d", date.da_mon, date.da_day, date.da_year);

  // load up config

  ResetMenu( FALSE );

}

void ResetMenu( BOOL InterBBSGame )
{
	char CurOption = 0, cInput, szString[50];
  BOOL Quit = FALSE, DidReset = FALSE, InterBBS = FALSE;
	int iTemp;

	/* clear all */
	clrscr();

	/* show options */
  qputs(" |03Reset for The Clans " VERSION, 0,0);
  qputs("|09- |01---------------------------------------------------------------------------",0,1);
	xputs(" Game start date", 0, 2);
	xputs(" Last date to join game", 0, 3);
	xputs(" Village Name", 0, 4);
	xputs(" Elimination mode", 0, 5);

	xputs(" Allow interBBS clan travel", 0, 6);
	xputs(" Days before lost troops/clans return", 0, 7);
	xputs(" Allow clan empires", 0, 8);
	xputs(" Mine fights per day", 0, 9);
	xputs(" Clan fights per day", 0, 10);

	xputs(" Max. Permanent Clan Members", 0, 11);
	xputs(" Days of Protection for new users", 0, 12);

	xputs(" Reset local game with these settings", 0, 13);
	xputs(" Join a league", 0, 14);
	xputs(" Leaguewide reset with these settings", 0, 15);
	xputs(" Abort reset", 0, 16);
  qputs("|01--------------------------------------------------------------------------- |09-",0,17);

  qputs("|09 Press the up and down keys to navigate.", 0, 18);

	/* init defaults */
  strcpy(ResetData.szDateGameStart, szTodaysDate);
  strcpy(ResetData.szLastJoinDate,  "12/16/2199");
	strcpy(ResetData.szVillageName,  "The Village");
	ResetData.InterBBSGame = InterBBSGame;
	ResetData.LeagueWide = FALSE;
	ResetData.InProgress = FALSE;
	ResetData.EliminationMode = FALSE;
	ResetData.ClanTravel = TRUE;
	ResetData.LostDays = 3;
	ResetData.ClanEmpires = TRUE;
  ResetData.MineFights = FIGHTSPERDAY;
	ResetData.ClanFights = CLANCOMBATADAY;
	ResetData.DaysOfProtection = 4;
	ResetData.MaxPermanentMembers = 4;

	while (!Quit)
	{
		/* show data */
    xputs(ResetData.szDateGameStart, 40, 2);
    xputs(ResetData.szLastJoinDate, 40, 3);
		xputs("                                        ", 40, 4);
		xputs(ResetData.szVillageName, 40, 4);
		if (ResetData.EliminationMode)
			xputs("Yes", 40, 5);
		else
			xputs("No ", 40, 5);

		if (ResetData.ClanTravel)
			xputs("Yes", 40, 6);
		else
			xputs("No ", 40, 6);

		sprintf(szString, "%-2d ", ResetData.LostDays);
		xputs(szString, 40, 7);

		if (ResetData.ClanEmpires)
			xputs("Yes", 40, 8);
		else
			xputs("No ", 40, 8);

		sprintf(szString, "%-2d ", ResetData.MineFights);
		xputs(szString, 40, 9);

		sprintf(szString, "%-2d ", ResetData.ClanFights);
		xputs(szString, 40, 10);


		sprintf(szString, "%-2d ", ResetData.MaxPermanentMembers);
		xputs(szString, 40, 11);

		sprintf(szString, "%-2d ", ResetData.DaysOfProtection);
		xputs(szString, 40, 12);

		/* dehilight all options */
		ColorArea(39,2, 76, 16, 15);	// choices on the right side

		/* dehilight options which can't be activated */
    if (Config->InterBBS == FALSE)
		{
			ColorArea(0,2, 39, 13, 7);
			ColorArea(0,14, 76, 15,  8);
		}
    else if (Config->BBSID != 1)
		{
			// not LC, can't use leaguewide reset option
			ColorArea(0,2, 39, 16, 7);
			ColorArea(0,15, 76, 15, 8);
			ColorArea(0,13, 39, 13, 8);
		}
		else
    {
      // LC's BBS AND IBBS set
      ColorArea(0,2, 39, 16, 7);
      ColorArea(0,13, 39, 14, 8);
    }

		ColorArea(0,2+MAX_OPTION, 39, 2+MAX_OPTION, 7);
		gotoxy(2, CurOption+3);

		/* show which is highlighted */
    ColorArea(0, CurOption+2, 76, CurOption+2, HILIGHT);

		cInput = getch();
		if (cInput == 0)
		{
			cInput = getch();

			switch ( cInput )
			{
				case K_UP	:
				case K_LEFT :
					if (CurOption == 0)
						CurOption = MAX_OPTION;
					else
						CurOption--;
					break;
				case K_DOWN :
				case K_RIGHT:
					if (CurOption == MAX_OPTION)
						CurOption = 0;
					else
						CurOption++;
					break;
				case K_HOME :
				case K_PGUP :
					CurOption = 0;
					break;
				case K_END :
				case K_PGDN :
					CurOption = MAX_OPTION;
					break;
			}
		}
		else if (cInput == 0x1B)
			Quit = TRUE;
		else if (cInput == 13)
		{
      if (Config->InterBBS == FALSE && CurOption >= (MAX_OPTION-2) &&
				CurOption < MAX_OPTION)
				continue;
      else if (Config->BBSID == 1 && Config->InterBBS && CurOption == (MAX_OPTION-2))
        continue;
      else if (Config->BBSID != 1 && CurOption == (MAX_OPTION-1))
			{
				continue;
			}


			if (CurOption == MAX_OPTION)
			{
				Quit = TRUE;
				DidReset = FALSE;
			}
			else if (CurOption == (MAX_OPTION-3))
			{
				// can't reset locally here
        if (Config->InterBBS)
					continue;

				/* reset local now */
				// ask one last time, are you sure?
				clrscr();
        if (YesNo("|07Are you sure you wish to reset?") == NO)
				{
					break;
				}

        GoReset(&ResetData, RESET_LOCAL);
				Quit = TRUE;
				DidReset = TRUE;
			}
			else if (CurOption == (MAX_OPTION-2))
			{
				// ask one last time, are you sure?
				clrscr();
				if (YesNo("|07Are you sure you are joining a league?") == NO)
				{
					break;
				}

				/* join league NOT in progress */
				ResetData.LeagueWide = TRUE;
				ResetData.InProgress = FALSE;
        GoReset(&ResetData, RESET_JOINIBBS);
				Quit = TRUE;
				DidReset = TRUE;
        InterBBS = TRUE;
			}
			else if (CurOption == (MAX_OPTION-1))
			{
				// ask one last time, are you sure?
				clrscr();
				if (YesNo("|07Are you sure you wish to reset the league?") == NO)
				{
					break;
				}

				/* leaguewide reset now */
				ResetData.LeagueWide = TRUE;
				GoReset(&ResetData, RESET_LC);
				Quit = TRUE;
				DidReset = TRUE;
        InterBBS = TRUE;
			}
			else
        EditOption(CurOption);
		}
	}

	clrscr();

	if (DidReset)
  {
    qputs("|09> |07Your game has been reset.", 0,0);
    if (InterBBS)
      qputs("|09> |03You must now type |11CLANS /I |03to complete the IBBS reset.", 0,1);
  }
	else
    qputs("|09> |07The game was not reset.  No changes were made.", 0,0);

  gotoxy(1,3);
}


  void CheckMem ( void *Test )
    /*
     * Gives system error if the pointer is NULL.
     */
  {
    if (Test == NULL)
    {
      printf("Checkmem Failed.\n");
      exit(0);
    }
  }

// ------------------------------------------------------------------------- //

  void qputs (char *string, int x, int y)
  {
    char number[3];
    int cur_char, attr;
    char foreground, background, cur_attr;
    static o_fg = 7, o_bg = 0;
    int i, j;

    cur_attr = o_fg | o_bg;
    cur_char=0;

    while (1) {
      if (string[cur_char] == 0)
        break;
      if (x == 80)
        break;
      if (string[cur_char]=='|')    {
        if ( isdigit(string[cur_char+1]) )  {
          number[0]=string[cur_char+1];
          number[1]=string[cur_char+2];
          number[2]=0;

          attr=atoi(number);
          if (attr>15)  {
            background=attr-16;
            o_bg = background << 4;
            attr = o_bg | o_fg;
            cur_attr = attr;
          }
          else  {
            foreground=attr;
            o_fg=foreground;
            attr = o_fg | o_bg;
            cur_attr = attr;
          }
          cur_char += 3;
        }
        else  {
          Video.VideoMem[(long) (Video.y_lookup[(long) y]+ (long) (x<<1)) ] = string[cur_char];
          Video.VideoMem[(long) (Video.y_lookup[(long) y]+ (long) (x<<1) + 1L)] = cur_attr;

          cur_char++;
          x++;
        }
      }
      else if (string[cur_char] == '\n')  {
        y++;
        cur_char++;
        x = 0;
      }
      else if (string[cur_char] == 9)  {  // tab
        cur_char++;
        for (i=0;i<8;i++) {
          j = i+x;
          if (j > 80) break;
          Video.VideoMem[ Video.y_lookup[y]+(j<<1) ] = ' ';
          Video.VideoMem[ Video.y_lookup[y]+(j<<1) + 1] = cur_attr;
        }
        x += 8;
      }
      else  {
        Video.VideoMem[ Video.y_lookup[y]+(x<<1) ] = string[cur_char];
        Video.VideoMem[ Video.y_lookup[y]+(x<<1) + 1] = cur_attr;

        cur_char++;
        x++;
      }
    }
  }



// ------------------------------------------------------------------------- //
  long Video_VideoType ( void )
  {
    return Video.VideoType;
  }

// ------------------------------------------------------------------------- //

  char far *vid_address ( void )
  {
    int tmp1, tmp2;

    asm {
      mov bx,0040h
      mov es,bx
      mov dx,es:63h
      add dx,6
      mov tmp1, dx           // move this into status port
    };
    Video.VideoType = COLOR;

    asm {
      mov bx,es:10h
      and bx,30h
      cmp bx,30h
      jne FC1
    };
    Video.VideoType = MONO;

    FC1:

    if (Video.VideoType == MONO)
      return ( ( void far * ) 0xB0000000L) ;
    else
      return ( ( void far * ) 0xB8000000L) ;

  }

  void ScrollUp ( void )
  {
    asm {
      mov al,1    // number of lines
      mov ch,0    // starting row
      mov cl,0    // starting column
      mov dh, 24  // ending row
      mov dl, 79  // ending column
      mov bh, 7   // color
      mov ah, 6
      int 10h     // do the scroll
    }
  }

// ------------------------------------------------------------------------- //

  void Video_Init ( void )
  {
    int iTemp;

    Video.VideoMem = vid_address();
    for (iTemp = 0; iTemp < 25;  iTemp++)
      Video.y_lookup[ iTemp ] = iTemp * 160;
  }

  void xputs (char *string, int x, int y)
  {
    char far *VidPtr;

    VidPtr = Video.VideoMem + Video.y_lookup[y] + (x<<1);

    while (x < 80 && *string)
    {
      *VidPtr = *string;

      VidPtr += 2;
      x++;
      string++;
    }
  }


int YesNo( char *Query )
{
	/* show query */
  zputs(Query);

	/* show Yes/no */
  zputs(" |01(|15Yes|07/no|01) |11");

	/* get user input */
  if (get_answer("YN\r") == 'N')
	{
		/* user says NO */
    zputs("No\n");
		return (NO);
	}
	else	/* user says YES */
	{
    zputs("Yes\n");
		return (YES);
	}
}

int NoYes( char *Query )
{
	/* show query */
  zputs(Query);

  zputs(" |01(|07yes/|15No|01) |11");

	/* get user input */
  if (get_answer("YN\r") == 'Y')
	{
		/* user says YES */
    zputs("Yes\n");
		return (YES);
	}
	else	/* user says NO */
	{
    zputs("No\n");
		return (NO);
	}
}


  void zputs (char *string)
  {
	char number[3];
	int cur_char, attr;
	char foreground, background, cur_attr;
	int i, j,  x,y;
	struct text_info TextInfo;
	static char o_fg = 7, o_bg = 0;

	gettextinfo(&TextInfo);
	x = TextInfo.curx-1;
	y = TextInfo.cury-1;

	cur_attr = o_fg | o_bg;
    cur_char = 0;

    for (;;)
    {
      if (y == 25)
      {
        /* scroll line up */
        ScrollUp();
        y = 24;
      }
      if (string[cur_char] == 0)
        break;
      if (string[cur_char] == '\b')
      {
        x--;
        cur_char++;
        continue;
      }
      if (string[cur_char] == '\r')
      {
        x = 0;
        cur_char++;
        continue;
      }
      if (x == 80)
      {
        x = 0;
        y++;

        if (y == 25)
        {
          /* scroll line up */
          ScrollUp();
          y = 24;
        }
        break;
      }
      if (string[cur_char]=='|')
      {
        if ( isdigit(string[cur_char+1]) && isdigit(string[cur_char+2]) )
        {
          number[0]=string[cur_char+1];
          number[1]=string[cur_char+2];
          number[2]=0;

          attr=atoi(number);
          if (attr>15)
          {
            background=attr-16;
            o_bg = background << 4;
            attr = o_bg | o_fg;
            cur_attr = attr;
          }
          else
          {
            foreground=attr;
            o_fg=foreground;
            attr = o_fg | o_bg;
            cur_attr = attr;
          }
          cur_char += 3;
        }
        else
        {
          Video.VideoMem[(long) (Video.y_lookup[(long) y]+ (long) (x<<1)) ] = string[cur_char];
          Video.VideoMem[(long) (Video.y_lookup[(long) y]+ (long) (x<<1) + 1L)] = cur_attr;

          cur_char++;
        }
      }
      else if (string[cur_char] == '\n')  {
        y++;
        if (y == 25)
        {
          /* scroll line up */
          ScrollUp();
          y = 24;
        }
        cur_char++;
        x = 0;
      }
      else if (string[cur_char] == 9)
      {
        /* tab */
        cur_char++;
        for (i=0;i<8;i++)
        {
          j = i + x;
          if (j > 80) break;
          Video.VideoMem[ Video.y_lookup[y]+(j<<1) ] = ' ';
          Video.VideoMem[ Video.y_lookup[y]+(j<<1) + 1] = cur_attr;
        }
        x += 8;
      }
      else
      {
        Video.VideoMem[ Video.y_lookup[y]+(x<<1) ] = string[cur_char];
        Video.VideoMem[ Video.y_lookup[y]+(x<<1) + 1] = cur_attr;

        cur_char++;
        x++;
      }
    }
    gotoxy(x+1,y+1);
  }

char get_answer( char *szAllowableChars )
{
	char cKey;
	int iTemp;

	for (;;)
	{
		cKey = getch();

		/* see if allowable */
		for (iTemp = 0; iTemp < strlen(szAllowableChars); iTemp++)
		{
			if (toupper(cKey) == toupper(szAllowableChars[iTemp]))
				break;
		}

		if (iTemp < strlen(szAllowableChars))
			break;	/* found allowable key */
	}

	return (toupper(cKey));
}

void EditOption ( int WhichOption )
{
	char *acScreen;
	char *aszHelp[11] = {
		"Date Game Starts",
		"Last Join Date",
		"Village Name",
		"Elimination Mode",
		"Clan Travel",
		"Lost Troops",
		"Clan Empires",
		"Mine Fights",
		"Clan Combat",
		"Max Permanent Members",
		"Days Of Protection"
		};
	int Month, Day, Year;

	/* save screen */
	acScreen = malloc(4000);
  memcpy(acScreen, Video.VideoMem, 4000);

	clrscr();

	/* show help */
	DosHelp(aszHelp[WhichOption], "RESET.HLP");


	switch ( WhichOption )
	{
		case 0 :	/* dategame started */
			gotoxy(1,15);
      Month = DosGetLong("|07Enter Month", atoi(ResetData.szDateGameStart), 12);
      Day = DosGetLong("|07Enter Day", atoi(&ResetData.szDateGameStart[3]), 31);
      Year = DosGetLong("|07Enter Year", atoi(&ResetData.szDateGameStart[6]), 2500);

      sprintf(ResetData.szDateGameStart, "%02d/%02d/%4d", Month, Day, Year);
			break;
		case 1 :	/* last join date */
			gotoxy(1,15);
      Month = DosGetLong("|07Enter Month", atoi(ResetData.szLastJoinDate), 12);
      Day = DosGetLong("|07Enter Day", atoi(&ResetData.szLastJoinDate[3]), 31);
      Year = DosGetLong("|07Enter Year", atoi(&ResetData.szLastJoinDate[6]), 2500);

      sprintf(ResetData.szLastJoinDate, "%02d/%02d/%4d", Month, Day, Year);
			break;
		case 2 :	/* village name */
			gotoxy(1,15);
			zputs("|07Please enter the village name|08> |15");
      DosGetStr(ResetData.szVillageName, 29);
			break;
		case 3 :	/* elimination mode? */
			gotoxy(1,15);
			if (YesNo("|07Setup elimination mode?") == YES)
			{
				ResetData.EliminationMode = TRUE;
			}
			else
				ResetData.EliminationMode = FALSE;
			break;
		case 4 :	/* clan travel? */
			gotoxy(1,15);
			if (YesNo("|07Allow clans to travel?") == YES)
			{
				ResetData.ClanTravel = TRUE;
			}
			else
				ResetData.ClanTravel = FALSE;
			break;
		case 5 :	/* days before lost troops return */
			gotoxy(1,15);
			ResetData.LostDays = DosGetLong("|07Days before lost troops/clans returned?", ResetData.LostDays, 14);
			break;
		case 6 :	/* clan empires? */
			gotoxy(1,15);
			if (YesNo("|07Allow clans to create empires?") == YES)
			{
				ResetData.ClanEmpires = TRUE;
			}
			else
				ResetData.ClanEmpires = FALSE;
			break;
		case 7 :	/* mine fights */
			gotoxy(1,15);
			ResetData.MineFights = DosGetLong("|07How many mine fights per day?", ResetData.MineFights, 50);
			break;
		case 8 :	/* clan combat */
			gotoxy(1,15);
			ResetData.ClanFights = DosGetLong("|07How many clan fights per day?", ResetData.ClanFights, MAX_CLANCOMBAT);
			break;
		case 9 :	/* max permanent clan members */
			gotoxy(1,15);
			ResetData.MaxPermanentMembers = DosGetLong("|07How many members?", ResetData.MaxPermanentMembers, 6);
			if (ResetData.MaxPermanentMembers == 0)
				ResetData.MaxPermanentMembers = 1;
			break;
		case 10:	/* days of protection */
			gotoxy(1,15);
			ResetData.DaysOfProtection = DosGetLong("|07How many days?", ResetData.DaysOfProtection, 16);
			break;
	}

	/* restore screen */
  memcpy(Video.VideoMem, acScreen, 4000);
	free(acScreen);
}

void DosHelp ( char *Topic, char *File )
{
	char *Lines[22], string[155];
	int cTemp, Found = FALSE, CurLine, NumLines;
	FILE *fp;
	BOOL EndOfTopic = FALSE;

	qputs("|15", 0, 0);
	qputs(Topic, 1, 0);
  qputs("|09- |01---------------------------------------------------------------------------",0,1);

  qputs("|01--------------------------------------------------------------------------- |09-",0,12);

	fp = fopen(File, "r");
	if (!fp)
	{
		qputs("|12Game help not found!\n", 0, 2);
		return;
	}

	for (cTemp = 0; cTemp < 22; cTemp++)
        Lines[cTemp] = malloc(255);

	/* search for topic */
	while (!Found)
	{
		if (fgets(string, 155, fp) == NULL)
		{
			fclose(fp);
			qputs("|12Game help not found!\n", 0, 2);
			for (cTemp = 0; cTemp < 22; cTemp++)
				free(Lines[cTemp]);
			return;
		}

		if (string[0] == '^')
		{
			/* see if topic is correct */
			if (strspn(&string[1], Topic) == strlen(Topic))
			{
				Found = TRUE;
			}
		}
	}

	while (EndOfTopic == FALSE)
	{
		/* read in up to 22 lines */
		for (CurLine = 0; CurLine < 22; CurLine++)
		{
			fgets(Lines[CurLine], 355, fp);

			if (Lines[CurLine][0] == '^')
			{
				if (strspn(&Lines[CurLine][1], "END") == 3)
				{
					EndOfTopic = TRUE;
					break;
				}
			}
		}
		NumLines = CurLine;

		/* display it */
		for (CurLine = 0; CurLine < NumLines; CurLine++)
		{
			qputs(Lines[CurLine], 0, CurLine+2);
		}
	}

	fclose(fp);

	for (cTemp = 0; cTemp < 22; cTemp++)
		free(Lines[cTemp]);

}

long DosGetLong( char *Prompt, long DefaultVal, long Maximum )
{
	char string[255], NumString[13], InputChar, DefMax[40];
	int NumDigits, CurDigit = 0, cTemp;
	long TenPower;

	/* init screen */
	zputs(" ");
	zputs(Prompt);

  sprintf(DefMax, " |01(|15%ld|07; %ld|01) |11", DefaultVal, Maximum);
	zputs(DefMax);

	/* NumDigits contains amount of digits allowed using max. value input */

	TenPower = 10;
	for (NumDigits = 1; NumDigits < 11; NumDigits++)
	{
		if (Maximum < TenPower )
			break;

		TenPower *= 10;
	}

	/* now get input */
	for (;;)
	{
		InputChar = get_answer("0123456789><.,\r\b\x19");

		if (isdigit(InputChar))
		{
			if (CurDigit < NumDigits)
			{
				NumString[CurDigit++] = InputChar;
				NumString[CurDigit] = 0;
				sprintf(string, "%c", InputChar);
				zputs(string);
			}

		}
		else if (InputChar == '\b')
		{
			if (CurDigit > 0)
			{
				CurDigit--;
				NumString[CurDigit] = 0;
				zputs("\b \b");
			}
		}
		else if (InputChar == '>' || InputChar == '.')
		{
			/* get rid of old value, by showing backspaces */
			for (cTemp = 0; cTemp < CurDigit; cTemp++)
				zputs("\b \b");

			sprintf(string, "%-ld", Maximum);
			string[NumDigits] = 0;
			zputs(string);

			strcpy(NumString, string);
			CurDigit = NumDigits;
		}
		else if (InputChar == '<' || InputChar == ',' || InputChar == 25)
		{
			/* get rid of old value, by showing backspaces */
			for (cTemp = 0; cTemp < CurDigit; cTemp++)
				zputs("\b \b");

			CurDigit = 0;
			string[CurDigit] = 0;

			strcpy(NumString, string);
		}
		else if (InputChar == '\r')
		{
			if (CurDigit == 0)
			{
				sprintf(string, "%-ld", DefaultVal);
				string[NumDigits] = 0;
				zputs(string);

				strcpy(NumString, string);
				CurDigit = NumDigits;

				zputs("\n");
				break;
			}
			/* see if number too high, if so, make it max */
			else if (atol(NumString) > Maximum)
			{
				/* get rid of old value, by showing backspaces */
				for (cTemp = 0; cTemp < CurDigit; cTemp++)
					zputs("\b \b");

				sprintf(string, "%-ld", Maximum);
				string[NumDigits] = 0;
				zputs(string);

				strcpy(NumString, string);
				CurDigit = NumDigits;
			}
			else	/* is a valid value */
			{
				zputs("\n");
				break;
			}
		}
	}

	return (atol(NumString));
}


void DosGetStr(char *InputStr, int MaxChars)
{
	int CurChar;
	unsigned char InputCh;
	char Spaces[85] = "                                                                                     ";
	char BackSpaces[85] = "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b";
	char TempStr[100], szString[100];

	Spaces[MaxChars] = 0;
	BackSpaces[MaxChars] = 0;

	CurChar = strlen(InputStr);

	zputs(Spaces);
	zputs(BackSpaces);
	zputs(InputStr);

	for (;;)
	{
		InputCh = getch();

		if (InputCh == '\b')
		{
			if (CurChar>0)
			{
				CurChar--;
				zputs("\b \b");
			}
		}
		else if (InputCh == '\r')
		{
			zputs("|16\n");
			InputStr[CurChar]=0;
			break;
		}
		else if (InputCh== '' || InputCh == '\x1B')  // ctrl-y
		{
			InputStr [0] = 0;
			strcpy(TempStr, BackSpaces);
			TempStr[ CurChar ] = 0;
			zputs(TempStr);
			Spaces[MaxChars] = 0;
			BackSpaces[MaxChars] = 0;
			zputs(Spaces);
			zputs(BackSpaces);
			CurChar = 0;
		}
		else if (InputCh >= '')
			continue;
		else if (InputCh == 0)
			continue;
		else if (iscntrl(InputCh))
			continue;
		else if (isalpha(InputCh) && CurChar && InputStr[CurChar-1] == SPECIAL_CODE)
			continue;
		else	/* valid character input */
		{
			if (CurChar==MaxChars)	 continue;
			InputStr[CurChar++]=InputCh;
			InputStr[CurChar] = 0;
			sprintf(szString, "%c", InputCh);
			zputs(szString);
		}
	}
}

void ColorArea( int xPos1, int yPos1, int xPos2, int yPos2, char Color )
{
	int x, y;

	for (y = yPos1; y <= yPos2;  y++)
	{
		for (x = xPos1;  x <= xPos2;  x++)
		{
      Video.VideoMem[ Video.y_lookup[y] + (x<<1) +1 ] = Color;
		}
	}
}

  void Config_Init ( void )
    /*
     * Loads data from .CFG file into Config->
     *
     */
  {
    FILE *fpConfigFile;
    char szConfigName[40], szConfigLine[255];
    char *pcCurrentPos;
    char szToken[MAX_TOKEN_CHARS + 1];
    int iKeyWord, iCurrentNode = 1;

    Config = malloc(sizeof(struct config));

    // --- Set defaults
    strcpy(szConfigName, "CLANS.CFG");
    Config->szSysopName[0] = 0;
    Config->szBBSName[0] = 0;
    strcpy(Config->szScoreFile[0], "SCORES.ASC");
    strcpy(Config->szScoreFile[1], "SCORES.ANS");
    Config->szRegcode[0] = 0;

    Config->InterBBS = FALSE;


    fpConfigFile = _fsopen(szConfigName, "rt", SH_DENYWR);
    if (!fpConfigFile)
    {
      /* file not found! error */
      printf("Config file not found.  Please run CONFIG.EXE first before resetting.\n");
      exit(0);
    }

    for (;;)
    {
      /* read in a line */
      if (fgets(szConfigLine, 255, fpConfigFile) == NULL) break;

      /* Ignore all of line after comments or CR/LF char */
      pcCurrentPos=(char *)szConfigLine;
      ParseLine(pcCurrentPos);

      /* If no token was found, proceed to process the next line */
      if(!*pcCurrentPos) continue;

      GetToken(pcCurrentPos, szToken);

      /* Loop through list of keywords */
      for(iKeyWord = 0; iKeyWord < MAX_CONFIG_WORDS; ++iKeyWord)
      {
        /* If keyword matches */
        if(stricmp(szToken, papszConfigKeyWords[iKeyWord]) == 0)
        {
          /* Process config token */
          switch (iKeyWord)
          {
            case 0 :  /* sysopname */
              strcpy(Config->szSysopName, pcCurrentPos);
              break;
            case 1 :  /* bbsname */
              strcpy(Config->szBBSName, pcCurrentPos);
              break;
            case 10 : /* BBS Id */
              Config->BBSID = atoi(pcCurrentPos);
              break;
            case 11 : /* netmail dir */
              strcpy(Config->szNetmailDir, pcCurrentPos);

              /* remove '\' if last char is it */
              if (Config->szNetmailDir [ strlen(Config->szNetmailDir) - 1] == '\\')
                Config->szNetmailDir [ strlen(Config->szNetmailDir) - 1] = 0;
              break;
            case 12 : /* inbound dir */
              strcpy(Config->szInboundDir, pcCurrentPos);

              /* add '\' if last char is not it */
              if ( Config->szInboundDir [ strlen(Config->szInboundDir) - 1] != '\\')
                strcat(Config->szInboundDir, "\\");
              break;
            case 13 : /* mailer type */
              if (stricmp(pcCurrentPos, "BINKLEY") == 0)
                Config->MailerType = MAIL_BINKLEY;
              else
                Config->MailerType = MAIL_OTHER;
              break;
            case 14 : /* in a league? */
              Config->InterBBS = TRUE;
              break;
          }
        }
      }
    }

    fclose(fpConfigFile);

  }

  void Config_Close ( void )
    /*
     * Shuts down config's mem.
     *
     */
  {
    free(Config);
  }


  void GoReset ( struct ResetData *ResetData, int Option )
  {
    FILE *fp;
    struct game_data Game_Data;
    struct time t;
    char szString[128];

    // Delete unwanted files here
    unlink("CLANS.MSJ");
    unlink("CLANS.PC");
    unlink("disband.dat");
    unlink("trades.dat");
    unlink("public.txt");
    unlink("pawn.dat");
    unlink("clans.npx");

    KillAlliances();

    unlink("ally.dat");

		// delete IBBS files
		unlink("IBBS.DAT");
		unlink("ipscores.dat");
		unlink("userlist.dat");
    unlink("backup.dat");
		unlink("Leaving.dat");

    if (Option == RESET_LOCAL)
    {
      /*
       * reset local game, no IBBS
       */

      // create new game.dat file
      Game_Data.GameState = 1;
      Game_Data.InterBBS = FALSE;

      strcpy(Game_Data.szTodaysDate, szTodaysDate);
      strcpy(Game_Data.szDateGameStart, ResetData->szDateGameStart);
      strcpy(Game_Data.szLastJoinDate, ResetData->szLastJoinDate);

      Game_Data.NextClanID = 0;
      Game_Data.NextAllianceID = 0;

      Game_Data.MaxPermanentMembers = ResetData->MaxPermanentMembers;
      Game_Data.ClanEmpires = ResetData->ClanEmpires;
      Game_Data.MineFights = ResetData->MineFights;
      Game_Data.ClanFights = ResetData->ClanFights;
      Game_Data.DaysOfProtection = ResetData->DaysOfProtection;


      Game_Data.CRC = CRCValue(&Game_Data, sizeof(struct game_data) - sizeof(long));

      fp = fopen("game.dat", "wb");
      // fwrite(&Game_Data, sizeof(struct game_data), 1, fp);
      EncryptWrite(&Game_Data, sizeof(struct game_data), fp, XOR_GAME);
      fclose(fp);

      // update news
      unlink("YEST.ASC");
      rename("TODAY.ASC", "YEST.ASC");
      News_CreateTodayNews();
      News_AddNews("|0A  |0CThe game has been reset!\n\n");
    }
    else if (Option == RESET_JOINIBBS)
    {
      // create new game.dat file
      Game_Data.GameState = 2;
      Game_Data.InterBBS = TRUE;

      /* find out date and time for GameId */
      // initialize date
      gettime(&t);
      sprintf(szString, "%02d:%02d:%02d.%02d", t.ti_hour, t.ti_min, t.ti_sec, t.ti_hund);
      strcpy(Game_Data.GameID, szString);

      strcpy(Game_Data.szTodaysDate, szTodaysDate);
      strcpy(Game_Data.szDateGameStart, ResetData->szDateGameStart);
      strcpy(Game_Data.szLastJoinDate, ResetData->szLastJoinDate);

      Game_Data.NextClanID = 0;
      Game_Data.NextAllianceID = 0;

      Game_Data.MaxPermanentMembers = ResetData->MaxPermanentMembers;
      Game_Data.ClanEmpires = ResetData->ClanEmpires;
      Game_Data.MineFights = ResetData->MineFights;
      Game_Data.ClanFights = ResetData->ClanFights;
      Game_Data.DaysOfProtection = ResetData->DaysOfProtection;


      Game_Data.CRC = CRCValue(&Game_Data, sizeof(struct game_data) - sizeof(long));

      fp = fopen("game.dat", "wb");
      EncryptWrite(&Game_Data, sizeof(struct game_data), fp, XOR_GAME);
      fclose(fp);

      // update news
      unlink("YEST.ASC");
      rename("TODAY.ASC", "YEST.ASC");
      News_CreateTodayNews();
      News_AddNews("|0A  |0CThe game is waiting for LC OK.\n\n");

    }
    else if (Option == RESET_LC)
    {
      // leaguewide reset

      // REP: make sure only LC can do this somehow...

      // create game.dat file
      Game_Data.GameState = 1;
      Game_Data.InterBBS = TRUE;

      /* find out date and time for GameId */
      gettime(&t);
      sprintf(szString, "%02d:%02d:%02d.%02d", t.ti_hour, t.ti_min, t.ti_sec, t.ti_hund);
      strcpy(Game_Data.GameID, szString);

      strcpy(Game_Data.szTodaysDate, szTodaysDate);
      strcpy(Game_Data.szDateGameStart, ResetData->szDateGameStart);
      strcpy(Game_Data.szLastJoinDate, ResetData->szLastJoinDate);

      Game_Data.NextClanID = 0;
      Game_Data.NextAllianceID = 0;

      Game_Data.MaxPermanentMembers = ResetData->MaxPermanentMembers;
      Game_Data.ClanEmpires = ResetData->ClanEmpires;
      Game_Data.MineFights = ResetData->MineFights;
      Game_Data.ClanFights = ResetData->ClanFights;
      Game_Data.DaysOfProtection = ResetData->DaysOfProtection;


      Game_Data.CRC = CRCValue(&Game_Data, sizeof(struct game_data) - sizeof(long));

      fp = fopen("game.dat", "wb");
      EncryptWrite(&Game_Data, sizeof(struct game_data), fp, XOR_GAME);
      fclose(fp);

      // update news
      unlink("YEST.ASC");
      rename("TODAY.ASC", "YEST.ASC");
      News_CreateTodayNews();
      News_AddNews("|0A  |0CThe game has been reset!\n\n");
    }

    // create new village data
    CreateVillageDat(ResetData);
  }


  void News_CreateTodayNews  ( void )
  {
    /* this initializes the TODAY.ASC file */

    char szString[128];

    sprintf(szString, "|0CNews for |0B%s\n\n", szTodaysDate);
    News_AddNews(szString);

    /* give other info like increase in cost of etc. */
  }

  void News_AddNews(char *szString)
  {
    FILE *fpNewsFile;

    /* open news file */

    /* add to it */

    fpNewsFile = _fsopen("TODAY.ASC", "at", SH_DENYWR);

    fputs(szString, fpNewsFile);

    fclose(fpNewsFile);
  }

  void ClearFlags ( char *Flags )
  {
    int iTemp;

    for (iTemp = 0; iTemp < 8; iTemp++)
      Flags[iTemp] = 0;
  }

  void InitEmpire ( struct empire *Empire, int UserEmpire )
  {
    int iTemp;

    if (UserEmpire)
      Empire->Land = 100;
    else
      Empire->Land = 0;

    Empire->Points = 0;

    for (iTemp = 0; iTemp < MAX_BUILDINGS; iTemp++)
      Empire->Buildings[iTemp] = 0;

    Empire->WorkerEnergy = 100;

    // deal with army
    Empire->Army.Followers = 0;
    Empire->Army.Footmen = 0;
    Empire->Army.Axemen = 0;
    Empire->Army.Knights = 0;
    Empire->Army.Rating = 100;
    Empire->Army.Level = 0;
    Empire->SpiesToday = 0;
    Empire->AttacksToday = 0;
    Empire->LandDevelopedToday = 0;
    Empire->Army.Strategy.AttackLength = 10;
    Empire->Army.Strategy.DefendLength = 10;
    Empire->Army.Strategy.LootLevel    = 0;
    Empire->Army.Strategy.AttackIntensity = 50;
    Empire->Army.Strategy.DefendIntensity = 50;
  }



  void CreateVillageDat ( struct ResetData *ResetData )
  {
    FILE *fp;
    struct village_data Village_Data = {
      { 1, 9, 3, 1, 11, 7, 3, 2, 10, 2, 5, 3, 9, 5, 13, 1, 9, 15, 7, 0, 0, 8, 4,
        1, 8, 1 },
      -1
      };

    Village_Data.PublicMsgIndex = 1;
    strcpy(Village_Data.szName, ResetData->szVillageName);

    Village_Data.TownType = -1;      /* unknown for now */
    Village_Data.TaxRate = 0;      /* no taxes */
    Village_Data.InterestRate = 0;   /* 0% interest rate */
    Village_Data.GST = 0;        /* no gst */
    Village_Data.Empire.VaultGold = 45000L; /* some money to begin with */
    Village_Data.ConscriptionRate = 10;

    Village_Data.RulingClanId[0] = -1; /* no ruling clan yet */
    Village_Data.RulingClanId[1] = -1;

    Village_Data.szRulingClan[0] = 0;
    Village_Data.RulingDays = 0;
    Village_Data.GovtSystem = GS_DEMOCRACY;


    Village_Data.MarketLevel = 0;      /* weaponshop level */
    Village_Data.TrainingHallLevel = 0;
    Village_Data.ChurchLevel = 0;
    Village_Data.PawnLevel = 0;
    Village_Data.WizardLevel = 0;

    Village_Data.SetTaxToday = FALSE;
    Village_Data.SetInterestToday = FALSE;
    Village_Data.SetGSTToday = FALSE;
    Village_Data.SetConToday = FALSE;

    Village_Data.UpMarketToday = FALSE;
    Village_Data.UpTHallToday = FALSE;
    Village_Data.UpChurchToday = FALSE;
    Village_Data.UpPawnToday = FALSE;
    Village_Data.UpWizToday = FALSE;
    Village_Data.ShowEmpireStats = FALSE;

    ClearFlags(Village_Data.HFlags);
    ClearFlags(Village_Data.GFlags);

    InitEmpire(&Village_Data.Empire, FALSE);
    strcpy(Village_Data.Empire.szName, Village_Data.szName);
    Village_Data.Empire.Land = 500;      // village starts off with this
    Village_Data.Empire.Buildings[B_BARRACKS] = 10;
    Village_Data.Empire.Buildings[B_WALL] = 10;
    Village_Data.Empire.Buildings[B_TOWER] = 5;
    Village_Data.Empire.Buildings[B_STEELMILL] = 5;
    Village_Data.Empire.Buildings[B_BUSINESS] = 10;
    Village_Data.Empire.Army.Footmen = 100;
    Village_Data.Empire.Army.Axemen = 25;

    Village_Data.CostFluctuation = 5 - random(11);
    Village_Data.MarketQuality = MQ_AVERAGE;

    Village_Data.CRC = CRCValue(&Village_Data, sizeof(struct village_data) - sizeof(long));

    fp = fopen("village.dat", "wb");
    // fwrite(&Village_Data, sizeof(struct village_data), 1, fp);
    EncryptWrite(&Village_Data, sizeof(struct village_data), fp, XOR_VILLAGE);
    fclose(fp);
  }



  void GetAlliances( struct Alliance *Alliances[MAX_ALLIANCES])
  {
    FILE *fp;
    int iTemp;

    // init alliances as NULLs
    for (iTemp = 0; iTemp < MAX_ALLIANCES; iTemp++)
      Alliances[iTemp] = NULL;

    fp = fopen("ALLY.DAT", "rb");
    if (fp)
    {
      for (iTemp = 0; iTemp < MAX_ALLIANCES; iTemp++)
      {
        Alliances[iTemp] = malloc(sizeof(struct Alliance));
        CheckMem(Alliances[iTemp]);

        if (EncryptRead(Alliances[iTemp], sizeof(struct Alliance), fp, XOR_ALLIES) == 0)
        {
          // no more alliances to read in
          free(Alliances[iTemp]);
          Alliances[iTemp] = NULL;
          break;
        }
      }
      fclose(fp);
    }
  }


  void UpdateAlliances( struct Alliance *Alliances[MAX_ALLIANCES])
  {
    FILE *fp;
    int iTemp;

    fp = fopen("ALLY.DAT", "wb");
    if (fp)
    {
      for (iTemp = 0; iTemp < MAX_ALLIANCES; iTemp++)
      {
        if (Alliances[iTemp] == NULL)
          continue;

        EncryptWrite(Alliances[iTemp], sizeof(struct Alliance), fp, XOR_ALLIES);
      }
      fclose(fp);
    }
  }


void KillAlliances( void )
{
	struct Alliance *Alliances[MAX_ALLIANCES];
	char szFileName[13];
	int iTemp;

  GetAlliances(Alliances);

	// delete files
	// free up mem used by alliances
	for (iTemp = 0; iTemp < MAX_ALLIANCES; iTemp++)
		if (Alliances[iTemp])
		{
			sprintf(szFileName, "HALL%02d.TXT", Alliances[iTemp]->ID);
			unlink(szFileName);

			free(Alliances[iTemp]);
            Alliances[iTemp] = NULL;
		}

	// free up mem used by alliances
    for (iTemp = 0; iTemp < MAX_ALLIANCES; iTemp++)
		if (Alliances[iTemp])
			free(Alliances[iTemp]);

	// called to destroy ALLY.DAT and remove those pesky HALLxxyy.TXT files
	unlink("ALLY.DAT");
}

  char *MakeStr ( int length )
    /*
     * This returns a pointer to a malloc'd string of length length.
     */
  {
    char *pc;

    pc = malloc(sizeof(char)*length);
    CheckMem(pc);

    return pc;
  }

  void System_Error ( char *szErrorMsg )
    /*
     * purpose  To output an error message and close down the system.
     *          This SHOULD be run from anywhere and NOT fail.  It should
     *          be FOOLPROOF.
     */
  {
    qputs("|12Error: |07", 0,0);
    qputs(szErrorMsg, 0, 7);
    delay(1000);
    exit(0);
  }
